MODULE BoxListe EXPORTS Boxen;

(* Dieses Modul implementiert die Verwaltung einer Boxrangliste,
   wie in der Schnittstelle Boxen definiert, mit Hilfe einer 
   einfach verketteten Liste. 
   Autor           : Moritz Schnizler, Antje Nowack
   Umgebung        : PM3, Windows 95
   Erstellt        : 06.01.99  Letzte Aenderung: 15.01.2001 *)


IMPORT SIO;      (* Importiere Operationen fuer die Ein-/Ausgabe *)
IMPORT Boxer;  (* Importiere den ADT Boxer *)
IMPORT Assertion AS As;

TYPE RanglisteRef = REF RanglistenElement;
     
     RanglistenElement = RECORD
                           boxer  : Boxer.T;
                           naechster: RanglisteRef;
                         END;


VAR rangliste    : RanglisteRef; (* Anker der Club-Rangliste *)


PROCEDURE SucheBoxerRL(boxer: Boxer.T): RanglisteRef=
(* Sucht den gegebenen Boxer in der Rangliste und gibt einen Zeiger
   auf ihn zurueck. Der Zeiger hat den Wert NIL, wenn der Boxer nicht in der 
   Liste vorhanden ist. *)

VAR resRLRef: RanglisteRef;
    gefunden: BOOLEAN;

BEGIN
   gefunden := FALSE;
   resRLRef := rangliste;
   WHILE (resRLRef # NIL) AND NOT gefunden DO

     (* Pruefe, ob der referenzierte Boxer der gesuchte ist *)
     IF Boxer.Gleich(resRLRef^.boxer, boxer) THEN
       gefunden := TRUE;
     ELSE
       resRLRef := resRLRef^.naechster;
     END;

   END; (* WHILE *)

   RETURN resRLRef;
END SucheBoxerRL;


PROCEDURE SucheVorgaengerRL(boxer: Boxer.T): RanglisteRef=
(* Sucht den Vorgaenger des gegebenen Boxers in der Rangliste und gibt
   einen Zeiger auf ihn zurueck. Der Zeiger hat den Wert NIL, wenn der Boxer
   keinen Vorgaenger hat, d.h. wenn er das erste Element oder nicht in der 
   Liste ist. *)

VAR vorRLRef, aktRLRef: RanglisteRef;
    gefunden          : BOOLEAN;

BEGIN
   gefunden := FALSE;
   vorRLRef := NIL;    (* Es gibt keinen Vorgaenger des ersten Elements! *)
   aktRLRef := rangliste;
   WHILE (aktRLRef # NIL) AND NOT gefunden DO
     
     (* Pruefe, ob der referenzierte Boxer der gesuchte ist *)
     IF Boxer.Gleich(aktRLRef^.boxer, boxer) THEN
       gefunden := TRUE;
     ELSE
       vorRLRef := aktRLRef;
       aktRLRef := aktRLRef^.naechster; 
     END;

   END; (* WHILE *)

   (* Zeiger auf den Vorgaenger ist NIL, wenn Boxer nicht in der Liste ist *)
   IF NOT gefunden THEN vorRLRef := NIL; END;

   RETURN vorRLRef;
END SucheVorgaengerRL;


PROCEDURE VollRL(): BOOLEAN=
(* Prueft, ob noch Platz fuer wenigstens einen Boxer in der Rangliste ist. *)
BEGIN
  (* In der optimistischen Annahme wir haetten unbegrenzten Speicherplatz, ist
     die Rangliste nie voll *)
  RETURN FALSE;
END VollRL;


PROCEDURE BoxerExistiertRL(boxer: Boxer.T): BOOLEAN=
(* Prueft, ob der angegebene Boxer in der Rangliste enthalten ist. *)
BEGIN
  RETURN (SucheBoxerRL(boxer) # NIL);
END BoxerExistiertRL;


PROCEDURE EinfuegenBoxerRL(boxer: Boxer.T) RAISES{As.Violated}=
(* Fuegt den gegebenen Boxer am Ende der Rangliste ein. Vorbedingung:
   Der Boxer ist noch nicht in der Rangliste enthalten. *)

VAR ranglisteRef, aktRLRef: RanglisteRef;

BEGIN
   (* Vorbedindungen *)
   As.Require(NOT BoxerExistiertRL(boxer), "Der Boxer ist schon in der Liste enthalten!");
   As.Require(NOT VollRL(), "Die Liste ist voll!!!");
  (* Erzeuge neues Ranglisten-Element fuer den Boxer *)
  ranglisteRef := NEW(RanglisteRef);
  ranglisteRef^.boxer := boxer;
  ranglisteRef^.naechster := NIL;

  (* Fuege Element an Ende der Liste an *)  
  IF (rangliste = NIL) THEN
    (* Liste ist noch leer *)
    rangliste := ranglisteRef;
  ELSE
    (* Suche Ende der Liste *)
    aktRLRef := rangliste;
    WHILE (aktRLRef^.naechster # NIL) DO
      aktRLRef := aktRLRef^.naechster;
    END;
    (* Haenge Boxer ans Ende an *)
    aktRLRef^.naechster := ranglisteRef;
  END;
  (* Nachbedingung *)
  As.Ensure(BoxerExistiertRL(boxer), "Fehler beim Einfuegen des Boxers!! Der Boxer ist nicht in der Liste enthalten!");

END EinfuegenBoxerRL;


PROCEDURE LoescheBoxerRL(boxer: Boxer.T) RAISES{As.Violated}=
(* Loescht den gegebenen Boxer aus der Rangliste. Vorbedingung:
   Der Boxer ist in der Rangliste enthalten. *)      

VAR ranglisteRef, 
    vorgaengerRef: RanglisteRef;

BEGIN
   (* Vorbedindung *)
   As.Require(BoxerExistiertRL(boxer), "Der Boxer ist nicht in der Liste enthalten!");
  (* Suche den Boxer in der Liste *)
  ranglisteRef := SucheBoxerRL(boxer);

  IF (ranglisteRef # NIL) THEN
    
    (* Vorhandenen Boxer aus der Rangliste loeschen *)
    vorgaengerRef := SucheVorgaengerRL(boxer);
    IF vorgaengerRef = NIL THEN
      (* Erstes Element in Rangliste, daher kein Vorgaenger *)
      rangliste := ranglisteRef^.naechster;
    ELSE
      vorgaengerRef^.naechster := ranglisteRef^.naechster;
    END;
  END;
  (* Nachbedingung *)
  As.Ensure(NOT BoxerExistiertRL(boxer), "Fehler beim Loeschen! Der Boxer ist noch immer enthalten1");
END LoescheBoxerRL;


PROCEDURE AKommtHinterBInRL(aRef, bRef: RanglisteRef): BOOLEAN=
(* Prueft, ob das durch Zeiger aRef referenzierte Element hinter dem durch bRef 
   referenzierten in der Rangliste vorkommt *)

VAR ranglisteRef: RanglisteRef;
  
BEGIN
  ranglisteRef := aRef;
  WHILE (ranglisteRef # NIL) AND (ranglisteRef # bRef) DO
    ranglisteRef := ranglisteRef^.naechster;
  END;

  RETURN NOT (ranglisteRef = bRef);
END AKommtHinterBInRL;


PROCEDURE AktualisiereRL(gewinner, verlierer: Boxer.T) RAISES{As.Violated}=
(* Aendert die Rangfolge in der Rangliste entsprechend dem gegebenen 
   Boxergebnis (Gewinner, Verlierer) *)

VAR gewinnerRef, verliererRef      : RanglisteRef;
    vorVerliererRef: RanglisteRef;

BEGIN
   (* Vorbedindungen *)
  As.Require(BoxerExistiertRL(gewinner), "Der Gewinner ist nicht in der Liste enthalten!");
  As.Require(BoxerExistiertRL(verlierer), "Der Verlierer ist nicht in der Liste enthalten!");

  (* Suche zunaechst Gewinner und Verlierer in der Rangliste *)
  gewinnerRef := SucheBoxerRL(gewinner);
  verliererRef := SucheBoxerRL( verlierer);
     
  (* Pruefe, ob nicht Gewinner ohnehin vor Verlierer in der Rangliste! *)
  IF NOT AKommtHinterBInRL(gewinnerRef, verliererRef) THEN
    SIO.PutLine("Keine Veraenderung in der Rangliste!");
  ELSE

      (* Fuege Verlierer hinter Gewinner in Rangliste ein *)

      (* Suche die jeweiligen Vorgaenger in der Rangliste *)
      vorVerliererRef := SucheVorgaengerRL(verlierer);

      (* Entferne Verlierer aus ursprnglicher Position *)
      IF ( vorVerliererRef = NIL) THEN
        (* Kein Vorgaenger, da ganz am Anfang der Liste *)
        rangliste := verliererRef^.naechster;
      ELSE
        vorVerliererRef^.naechster := verliererRef^.naechster;
      END;

      (* Hnge Verlierer hinter Gewinner ein *)
      verliererRef^.naechster := gewinnerRef^.naechster;
      gewinnerRef^.naechster := verliererRef;     
       (* Nachbedingung *)
      As.Ensure(AKommtHinterBInRL(verliererRef, gewinnerRef), "Fehler beim Bearbeiten des Kampfes");
    END;
END AktualisiereRL;


PROCEDURE DruckeRL()=
(* Gibt die Rangliste in absteigender Rangfolge aus *)

VAR ranglisteRef: RanglisteRef;
    rang        : CARDINAL;

BEGIN
  (* Gib Kopfkommentar der Rangliste aus *)
  SIO.PutLine(" *** Box-Rangliste *** "); 
  SIO.Nl();
 
  ranglisteRef := rangliste;
  rang := 1;
  WHILE (ranglisteRef # NIL) DO

    (* Gib Rang und Daten des Boxers aus *)
    SIO.PutInt(rang); SIO.PutText(". ");
    Boxer.Drucke(ranglisteRef^.boxer);

    (* Inkrementiere Boxer und Rang *)
    ranglisteRef := ranglisteRef^.naechster;
    rang := rang + 1;

  END; (* WHILE *)
END DruckeRL;


BEGIN
  (* Initialisiere Liste mit leerer Rangliste *)
  rangliste := NIL;
END BoxListe.


























































